# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    ExternalMappingResponse,
)
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExternalMappings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
            integration_config_id="integrationConfigId",
            version=0,
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.create(
                org_id="",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_mappings.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
            integration_config_id="integrationConfigId",
            version=0,
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.update(
                id="id",
                org_id="",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_mappings.with_raw_response.update(
                id="",
                org_id="orgId",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list(
            org_id="orgId",
            external_system_id="externalSystemId",
            integration_config_id="integrationConfigId",
            m3ter_ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_mappings.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_list_by_external_entity(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_method_list_by_external_entity_with_all_params(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_raw_response_list_by_external_entity(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_list_by_external_entity(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_by_external_entity(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="",
                system="system",
                external_table="externalTable",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `system` but received ''"):
            client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="orgId",
                system="",
                external_table="externalTable",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_table` but received ''"):
            client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="orgId",
                system="system",
                external_table="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_id` but received ''"):
            client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="",
                org_id="orgId",
                system="system",
                external_table="externalTable",
            )

    @parametrize
    def test_method_list_by_m3ter_entity(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_method_list_by_m3ter_entity_with_all_params(self, client: M3ter) -> None:
        external_mapping = client.external_mappings.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_raw_response_list_by_m3ter_entity(self, client: M3ter) -> None:
        response = client.external_mappings.with_raw_response.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = response.parse()
        assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    def test_streaming_response_list_by_m3ter_entity(self, client: M3ter) -> None:
        with client.external_mappings.with_streaming_response.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = response.parse()
            assert_matches_type(SyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_by_m3ter_entity(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="m3terId",
                org_id="",
                entity="entity",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `entity` but received ''"):
            client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="m3terId",
                org_id="orgId",
                entity="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `m3ter_id` but received ''"):
            client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="",
                org_id="orgId",
                entity="entity",
            )


class TestAsyncExternalMappings:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
            integration_config_id="integrationConfigId",
            version=0,
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.create(
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.create(
                org_id="",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_mappings.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
            integration_config_id="integrationConfigId",
            version=0,
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.update(
            id="id",
            org_id="orgId",
            external_id="JS!?Q0]r] ]$]",
            external_system="JS!?Q0]r] ]$]",
            external_table="JS!?Q0]r] ]$]",
            m3ter_entity="JS!?Q0]r] ]$]",
            m3ter_id="JS!?Q0]r] ]$]",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.update(
                id="id",
                org_id="",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_mappings.with_raw_response.update(
                id="",
                org_id="orgId",
                external_id="JS!?Q0]r] ]$]",
                external_system="JS!?Q0]r] ]$]",
                external_table="JS!?Q0]r] ]$]",
                m3ter_entity="JS!?Q0]r] ]$]",
                m3ter_id="JS!?Q0]r] ]$]",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list(
            org_id="orgId",
            external_system_id="externalSystemId",
            integration_config_id="integrationConfigId",
            m3ter_ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(ExternalMappingResponse, external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_mappings.with_raw_response.delete(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_list_by_external_entity(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_method_list_by_external_entity_with_all_params(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_list_by_external_entity(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_list_by_external_entity(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.list_by_external_entity(
            external_id="externalId",
            org_id="orgId",
            system="system",
            external_table="externalTable",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_by_external_entity(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="",
                system="system",
                external_table="externalTable",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `system` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="orgId",
                system="",
                external_table="externalTable",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_table` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="externalId",
                org_id="orgId",
                system="system",
                external_table="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_id` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_external_entity(
                external_id="",
                org_id="orgId",
                system="system",
                external_table="externalTable",
            )

    @parametrize
    async def test_method_list_by_m3ter_entity(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_method_list_by_m3ter_entity_with_all_params(self, async_client: AsyncM3ter) -> None:
        external_mapping = await async_client.external_mappings.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_raw_response_list_by_m3ter_entity(self, async_client: AsyncM3ter) -> None:
        response = await async_client.external_mappings.with_raw_response.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_mapping = await response.parse()
        assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

    @parametrize
    async def test_streaming_response_list_by_m3ter_entity(self, async_client: AsyncM3ter) -> None:
        async with async_client.external_mappings.with_streaming_response.list_by_m3ter_entity(
            m3ter_id="m3terId",
            org_id="orgId",
            entity="entity",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_mapping = await response.parse()
            assert_matches_type(AsyncCursor[ExternalMappingResponse], external_mapping, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_by_m3ter_entity(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="m3terId",
                org_id="",
                entity="entity",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `entity` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="m3terId",
                org_id="orgId",
                entity="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `m3ter_id` but received ''"):
            await async_client.external_mappings.with_raw_response.list_by_m3ter_entity(
                m3ter_id="",
                org_id="orgId",
                entity="entity",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .permission_statement_response import PermissionStatementResponse

__all__ = ["PermissionPolicyResponse"]


class PermissionPolicyResponse(BaseModel):
    id: Optional[str] = None
    """The unique identifier (UUID) for this Permission Policy."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this Permission Policy."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the Permission Policy was created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the Permission Policy was last
    modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) of the user who last modified this Permission
    Policy.
    """

    managed_policy: Optional[bool] = FieldInfo(alias="managedPolicy", default=None)
    """Indicates whether this is a system generated Managed Permission Policy."""

    name: Optional[str] = None
    """The name of the Permission Policy."""

    permission_policy: Optional[List[PermissionStatementResponse]] = FieldInfo(alias="permissionPolicy", default=None)
    """Array containing the Permission Policies information."""

    version: Optional[int] = None
    """The version number. Default value when newly created is one."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .data_field_response_param import DataFieldResponseParam

__all__ = ["MeterUpdateParams", "DerivedField"]


class MeterUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    code: Required[str]
    """Code of the Meter - unique short code used to identify the Meter.

    **NOTE:** Code has a maximum length of 80 characters and must not contain
    non-printable or whitespace characters (except space), and cannot start/end with
    whitespace.
    """

    data_fields: Required[Annotated[Iterable[DataFieldResponseParam], PropertyInfo(alias="dataFields")]]
    """
    Used to submit categorized raw usage data values for ingest into the platform -
    either numeric quantitative values or non-numeric data values. At least one
    required per Meter; maximum 15 per Meter.
    """

    derived_fields: Required[Annotated[Iterable[DerivedField], PropertyInfo(alias="derivedFields")]]
    """
    Used to submit usage data values for ingest into the platform that are the
    result of a calculation performed on `dataFields`, `customFields`, or system
    `Timestamp` fields. Raw usage data is not submitted using `derivedFields`.
    Maximum 15 per Meter. _(Optional)_.

    **Note:** Required parameter. If you want to create a Meter without Derived
    Fields, use an empty array `[]`. If you use a `null`, you'll receive an error.
    """

    name: Required[str]
    """Descriptive name for the Meter."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    group_id: Annotated[str, PropertyInfo(alias="groupId")]
    """UUID of the group the Meter belongs to. _(Optional)_."""

    product_id: Annotated[str, PropertyInfo(alias="productId")]
    """UUID of the product the Meter belongs to.

    _(Optional)_ - if left blank, the Meter is global.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class DerivedField(DataFieldResponseParam, total=False):
    calculation: Required[str]
    """
    The calculation used to transform the value of submitted `dataFields` in usage
    data. Calculation can reference `dataFields`, `customFields`, or system
    `Timestamp` fields. _(Example: datafieldms datafieldgb)_
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["TransactionSummaryResponse"]


class TransactionSummaryResponse(BaseModel):
    initial_credit_amount: Optional[float] = FieldInfo(alias="initialCreditAmount", default=None)

    total_credit_amount: Optional[float] = FieldInfo(alias="totalCreditAmount", default=None)

    total_debit_amount: Optional[float] = FieldInfo(alias="totalDebitAmount", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CurrencyConversion"]


class CurrencyConversion(BaseModel):
    from_: str = FieldInfo(alias="from")
    """Currency to convert from. For example: GBP."""

    to: str
    """Currency to convert to. For example: USD."""

    multiplier: Optional[float] = None
    """Conversion rate between currencies."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .file_upload_generate_upload_url_params import (
    FileUploadGenerateUploadURLParams as FileUploadGenerateUploadURLParams,
)
from .file_upload_generate_upload_url_response import (
    FileUploadGenerateUploadURLResponse as FileUploadGenerateUploadURLResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ResourceGroupResponse"]


class ResourceGroupResponse(BaseModel):
    id: Optional[str] = None
    """The unique identifier (UUID) of the Resource Group."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this Resource Group."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the Resource Group was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the Resource Group was last
    modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this Resource Group."""

    name: Optional[str] = None
    """The name of the Resource Group."""

    version: Optional[int] = None
    """The version number. Default value when newly created is one."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CompoundAggregationResponse"]


class CompoundAggregationResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    accounting_product_id: Optional[str] = FieldInfo(alias="accountingProductId", default=None)

    calculation: Optional[str] = None
    """This field is a string that represents the formula for the calculation.

    This formula determines how the CompoundAggregation is calculated from the
    underlying usage data.
    """

    code: Optional[str] = None
    """Code of the Aggregation. A unique short code to identify the Aggregation."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this CompoundAggregation."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the CompoundAggregation was
    created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO-8601 format)_ when the CompoundAggregation was last
    modified.
    """

    evaluate_null_aggregations: Optional[bool] = FieldInfo(alias="evaluateNullAggregations", default=None)
    """This is a boolean True / False flag.

    If set to TRUE, the calculation will be evaluated even if the referenced
    aggregation has no usage data.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) of the user who last modified this
    CompoundAggregation.
    """

    name: Optional[str] = None
    """Descriptive name for the Aggregation."""

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """
    This field represents the unique identifier (UUID) of the Product that is
    associated with the CompoundAggregation.
    """

    quantity_per_unit: Optional[float] = FieldInfo(alias="quantityPerUnit", default=None)
    """Defines how much of a quantity equates to 1 unit.

    Used when setting the price per unit for billing purposes - if charging for
    kilobytes per second (KiBy/s) at rate of $0.25 per 500 KiBy/s, then set
    quantityPerUnit to 500 and price Plan at $0.25 per unit.

    If `quantityPerUnit` is set to a value other than one, rounding is typically set
    to UP.
    """

    rounding: Optional[Literal["UP", "DOWN", "NEAREST", "NONE"]] = None
    """
    Specifies how you want to deal with non-integer, fractional number Aggregation
    values.

    **NOTES:**

    - **NEAREST** rounds to the nearest half: 5.1 is rounded to 5, and 3.5 is
      rounded to 4.
    - Also used in combination with `quantityPerUnit`. Rounds the number of units
      after `quantityPerUnit` is applied. If you set `quantityPerUnit` to a value
      other than one, you would typically set Rounding to **UP**. For example,
      suppose you charge by kilobytes per second (KiBy/s), set `quantityPerUnit` =
      500, and set charge rate at $0.25 per unit used. If your customer used 48,900
      KiBy/s in a billing period, the charge would be 48,900 / 500 = 97.8 rounded up
      to 98 \\** 0.25 = $2.45.

    Enum: ???UP??? ???DOWN??? ???NEAREST??? ???NONE???
    """

    segments: Optional[List[Dict[str, str]]] = None
    """_(Optional)_.

    Used when creating a segmented Aggregation, which segments the usage data
    collected by a single Meter. Works together with `segmentedFields`.

    Contains the values that are to be used as the segments, read from the fields in
    the meter pointed at by `segmentedFields`.
    """

    unit: Optional[str] = None
    """User defined or following the _Unified Code for Units of Measure_ (UCUM).

    Used as the label for billing, indicating to your customers what they are being
    charged for.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import OrganizationConfigResponse
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOrganizationConfig:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        organization_config = client.organization_config.retrieve(
            org_id="orgId",
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.organization_config.with_raw_response.retrieve(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization_config = response.parse()
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.organization_config.with_streaming_response.retrieve(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization_config = response.parse()
            assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.organization_config.with_raw_response.retrieve(
                org_id="",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        organization_config = client.organization_config.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        organization_config = client.organization_config.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
            auto_approve_bills_grace_period=2,
            auto_approve_bills_grace_period_unit="DAYS",
            auto_generate_statement_mode="NONE",
            bill_prefix="Bill-",
            commitment_fee_bill_in_advance=True,
            consolidate_bills=True,
            credit_application_order=["PREPAYMENT"],
            currency_conversions=[
                {
                    "from": "EUR",
                    "to": "USD",
                    "multiplier": 1.12,
                }
            ],
            default_statement_definition_id="defaultStatementDefinitionId",
            external_invoice_date="LAST_DAY_OF_ARREARS",
            minimum_spend_bill_in_advance=True,
            scheduled_bill_interval=0,
            sequence_start_number=1000,
            standing_charge_bill_in_advance=True,
            suppressed_empty_bills=True,
            version=0,
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.organization_config.with_raw_response.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization_config = response.parse()
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.organization_config.with_streaming_response.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization_config = response.parse()
            assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.organization_config.with_raw_response.update(
                org_id="",
                currency="USD",
                day_epoch="2022-01-01",
                days_before_bill_due=1,
                month_epoch="2022-01-01",
                timezone="UTC",
                week_epoch="2022-01-04",
                year_epoch="2022-01-01",
            )


class TestAsyncOrganizationConfig:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        organization_config = await async_client.organization_config.retrieve(
            org_id="orgId",
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.organization_config.with_raw_response.retrieve(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization_config = await response.parse()
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.organization_config.with_streaming_response.retrieve(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization_config = await response.parse()
            assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.organization_config.with_raw_response.retrieve(
                org_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        organization_config = await async_client.organization_config.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        organization_config = await async_client.organization_config.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
            auto_approve_bills_grace_period=2,
            auto_approve_bills_grace_period_unit="DAYS",
            auto_generate_statement_mode="NONE",
            bill_prefix="Bill-",
            commitment_fee_bill_in_advance=True,
            consolidate_bills=True,
            credit_application_order=["PREPAYMENT"],
            currency_conversions=[
                {
                    "from": "EUR",
                    "to": "USD",
                    "multiplier": 1.12,
                }
            ],
            default_statement_definition_id="defaultStatementDefinitionId",
            external_invoice_date="LAST_DAY_OF_ARREARS",
            minimum_spend_bill_in_advance=True,
            scheduled_bill_interval=0,
            sequence_start_number=1000,
            standing_charge_bill_in_advance=True,
            suppressed_empty_bills=True,
            version=0,
        )
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.organization_config.with_raw_response.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        organization_config = await response.parse()
        assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.organization_config.with_streaming_response.update(
            org_id="orgId",
            currency="USD",
            day_epoch="2022-01-01",
            days_before_bill_due=1,
            month_epoch="2022-01-01",
            timezone="UTC",
            week_epoch="2022-01-04",
            year_epoch="2022-01-01",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            organization_config = await response.parse()
            assert_matches_type(OrganizationConfigResponse, organization_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.organization_config.with_raw_response.update(
                org_id="",
                currency="USD",
                day_epoch="2022-01-01",
                days_before_bill_due=1,
                month_epoch="2022-01-01",
                timezone="UTC",
                week_epoch="2022-01-04",
                year_epoch="2022-01-01",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["M3terSignedCredentialsRequest"]


class M3terSignedCredentialsRequest(BaseModel):
    api_key: str = FieldInfo(alias="apiKey")
    """The API key provided by m3ter.

    This key is part of the credential set required for signing requests and
    authenticating with m3ter services.
    """

    secret: str
    """The secret associated with the API key.

    This secret is used in conjunction with the API key to generate a signature for
    secure authentication.
    """

    type: Literal["M3TER_SIGNED_REQUEST"]
    """Specifies the authorization type.

    For this schema, it is exclusively set to M3TER_SIGNED_REQUEST.
    """

    empty: Optional[bool] = None
    """A flag to indicate whether the credentials are empty.

    - TRUE - empty credentials.
    - FALSE - credential details required.
    """

    version: Optional[int] = None
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ..types import (
    contract_list_params,
    contract_create_params,
    contract_update_params,
    contract_end_date_billing_entities_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.contract_response import ContractResponse
from ..types.contract_end_date_billing_entities_response import ContractEndDateBillingEntitiesResponse

__all__ = ["ContractsResource", "AsyncContractsResource"]


class ContractsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContractsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ContractsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContractsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ContractsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        end_date: Union[str, date],
        name: str,
        start_date: Union[str, date],
        code: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Create a new Contract.

        Creates a new Contract for the specified Account.

        The Contract includes
        information such as the associated Account along with start and end dates.

        Args:
          account_id: The unique identifier (UUID) of the Account associated with this Contract.

          end_date: The exclusive end date of the Contract _(in ISO-8601 format)_. This means the
              Contract is active until midnight on the day **_before_** this date.

          name: The name of the Contract.

          start_date: The start date for the Contract _(in ISO-8601 format)_. This date is inclusive,
              meaning the Contract is active from this date onward.

          code: The short code of the Contract.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: The description of the Contract, which provides context and information.

          purchase_order_number: The Purchase Order Number associated with the Contract.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/contracts",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "code": code,
                    "custom_fields": custom_fields,
                    "description": description,
                    "purchase_order_number": purchase_order_number,
                    "version": version,
                },
                contract_create_params.ContractCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Retrieves the Contract with the given UUID.

        Used to obtain the details of a
        Contract.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/contracts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        end_date: Union[str, date],
        name: str,
        start_date: Union[str, date],
        code: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """
        Update the Contract with the given UUID.

        This endpoint updates the details of the Contract with the specified ID. Used to
        modify details of an existing Contract such as the start or end dates.

        **Note:** If you have created Custom Fields for a Contract, when you use this
        endpoint to update the Contract use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          account_id: The unique identifier (UUID) of the Account associated with this Contract.

          end_date: The exclusive end date of the Contract _(in ISO-8601 format)_. This means the
              Contract is active until midnight on the day **_before_** this date.

          name: The name of the Contract.

          start_date: The start date for the Contract _(in ISO-8601 format)_. This date is inclusive,
              meaning the Contract is active from this date onward.

          code: The short code of the Contract.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: The description of the Contract, which provides context and information.

          purchase_order_number: The Purchase Order Number associated with the Contract.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/contracts/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "code": code,
                    "custom_fields": custom_fields,
                    "description": description,
                    "purchase_order_number": purchase_order_number,
                    "version": version,
                },
                contract_update_params.ContractUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: Optional[str] | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ContractResponse]:
        """Retrieves a list of Contracts by Organization ID.

        Supports pagination and
        includes various query parameters to filter the Contracts returned based on
        Contract IDs or short codes.

        Args:
          codes: An optional parameter to retrieve specific Contracts based on their short codes.

          ids: An optional parameter to filter the list based on specific Contract unique
              identifiers (UUIDs).

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Contracts in a paginated list.

          page_size: Specifies the maximum number of Contracts to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/contracts",
            page=SyncCursor[ContractResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    contract_list_params.ContractListParams,
                ),
            ),
            model=ContractResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Deletes the Contract with the specified UUID.

        Used to remove an existing
        Contract from an Account.

        **Note:** This call will fail if there are any AccountPlans or Commitments that
        have been added to the Contract.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/contracts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    def end_date_billing_entities(
        self,
        id: str,
        *,
        org_id: str | None = None,
        billing_entities: List[Literal["CONTRACT", "ACCOUNTPLAN", "PREPAYMENT", "PRICINGS", "COUNTER_PRICINGS"]],
        end_date: Union[str, datetime],
        apply_to_children: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractEndDateBillingEntitiesResponse:
        """
        Apply the specified end-date to billing entities associated with Accounts the
        Contract has been added to, and apply the end-date to the Contract itself.

        **NOTES:**

        - If you want to apply the end-date to the Contract _itself_ - the Contract `id`
          you use as the required PATH PARAMETER - you must also specify `CONTRACT` as a
          `billingEntities` option in the request body schema.
        - Only the Contract whose id you specify for the PATH PARAMETER will be
          end-dated. If there are other Contracts associated with the Account, these
          will not be end-dated.
        - When you successfully end-date billing entities, the version number of each
          entity is incremented.

        Args:
          billing_entities: Defines which billing entities associated with the Account will have the
              specified end-date applied. For example, if you want the specified end-date to
              be applied to all Prepayments/Commitments created for the Account use
              `"PREPAYMENT"`.

          end_date: The end date and time applied to the specified billing entities _(in ISO 8601
              format)_.

          apply_to_children: A Boolean TRUE/FALSE flag. For Parent Accounts, set to TRUE if you want the
              specified end-date to be applied to any billing entities associated with Child
              Accounts. _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/contracts/{id}/enddatebillingentities",
            body=maybe_transform(
                {
                    "billing_entities": billing_entities,
                    "end_date": end_date,
                    "apply_to_children": apply_to_children,
                },
                contract_end_date_billing_entities_params.ContractEndDateBillingEntitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEndDateBillingEntitiesResponse,
        )


class AsyncContractsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContractsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContractsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContractsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncContractsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        end_date: Union[str, date],
        name: str,
        start_date: Union[str, date],
        code: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Create a new Contract.

        Creates a new Contract for the specified Account.

        The Contract includes
        information such as the associated Account along with start and end dates.

        Args:
          account_id: The unique identifier (UUID) of the Account associated with this Contract.

          end_date: The exclusive end date of the Contract _(in ISO-8601 format)_. This means the
              Contract is active until midnight on the day **_before_** this date.

          name: The name of the Contract.

          start_date: The start date for the Contract _(in ISO-8601 format)_. This date is inclusive,
              meaning the Contract is active from this date onward.

          code: The short code of the Contract.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: The description of the Contract, which provides context and information.

          purchase_order_number: The Purchase Order Number associated with the Contract.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/contracts",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "code": code,
                    "custom_fields": custom_fields,
                    "description": description,
                    "purchase_order_number": purchase_order_number,
                    "version": version,
                },
                contract_create_params.ContractCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Retrieves the Contract with the given UUID.

        Used to obtain the details of a
        Contract.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/contracts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        end_date: Union[str, date],
        name: str,
        start_date: Union[str, date],
        code: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """
        Update the Contract with the given UUID.

        This endpoint updates the details of the Contract with the specified ID. Used to
        modify details of an existing Contract such as the start or end dates.

        **Note:** If you have created Custom Fields for a Contract, when you use this
        endpoint to update the Contract use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          account_id: The unique identifier (UUID) of the Account associated with this Contract.

          end_date: The exclusive end date of the Contract _(in ISO-8601 format)_. This means the
              Contract is active until midnight on the day **_before_** this date.

          name: The name of the Contract.

          start_date: The start date for the Contract _(in ISO-8601 format)_. This date is inclusive,
              meaning the Contract is active from this date onward.

          code: The short code of the Contract.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: The description of the Contract, which provides context and information.

          purchase_order_number: The Purchase Order Number associated with the Contract.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/contracts/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "code": code,
                    "custom_fields": custom_fields,
                    "description": description,
                    "purchase_order_number": purchase_order_number,
                    "version": version,
                },
                contract_update_params.ContractUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: Optional[str] | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ContractResponse, AsyncCursor[ContractResponse]]:
        """Retrieves a list of Contracts by Organization ID.

        Supports pagination and
        includes various query parameters to filter the Contracts returned based on
        Contract IDs or short codes.

        Args:
          codes: An optional parameter to retrieve specific Contracts based on their short codes.

          ids: An optional parameter to filter the list based on specific Contract unique
              identifiers (UUIDs).

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Contracts in a paginated list.

          page_size: Specifies the maximum number of Contracts to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/contracts",
            page=AsyncCursor[ContractResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    contract_list_params.ContractListParams,
                ),
            ),
            model=ContractResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractResponse:
        """Deletes the Contract with the specified UUID.

        Used to remove an existing
        Contract from an Account.

        **Note:** This call will fail if there are any AccountPlans or Commitments that
        have been added to the Contract.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/contracts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractResponse,
        )

    async def end_date_billing_entities(
        self,
        id: str,
        *,
        org_id: str | None = None,
        billing_entities: List[Literal["CONTRACT", "ACCOUNTPLAN", "PREPAYMENT", "PRICINGS", "COUNTER_PRICINGS"]],
        end_date: Union[str, datetime],
        apply_to_children: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractEndDateBillingEntitiesResponse:
        """
        Apply the specified end-date to billing entities associated with Accounts the
        Contract has been added to, and apply the end-date to the Contract itself.

        **NOTES:**

        - If you want to apply the end-date to the Contract _itself_ - the Contract `id`
          you use as the required PATH PARAMETER - you must also specify `CONTRACT` as a
          `billingEntities` option in the request body schema.
        - Only the Contract whose id you specify for the PATH PARAMETER will be
          end-dated. If there are other Contracts associated with the Account, these
          will not be end-dated.
        - When you successfully end-date billing entities, the version number of each
          entity is incremented.

        Args:
          billing_entities: Defines which billing entities associated with the Account will have the
              specified end-date applied. For example, if you want the specified end-date to
              be applied to all Prepayments/Commitments created for the Account use
              `"PREPAYMENT"`.

          end_date: The end date and time applied to the specified billing entities _(in ISO 8601
              format)_.

          apply_to_children: A Boolean TRUE/FALSE flag. For Parent Accounts, set to TRUE if you want the
              specified end-date to be applied to any billing entities associated with Child
              Accounts. _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/contracts/{id}/enddatebillingentities",
            body=await async_maybe_transform(
                {
                    "billing_entities": billing_entities,
                    "end_date": end_date,
                    "apply_to_children": apply_to_children,
                },
                contract_end_date_billing_entities_params.ContractEndDateBillingEntitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEndDateBillingEntitiesResponse,
        )


class ContractsResourceWithRawResponse:
    def __init__(self, contracts: ContractsResource) -> None:
        self._contracts = contracts

        self.create = to_raw_response_wrapper(
            contracts.create,
        )
        self.retrieve = to_raw_response_wrapper(
            contracts.retrieve,
        )
        self.update = to_raw_response_wrapper(
            contracts.update,
        )
        self.list = to_raw_response_wrapper(
            contracts.list,
        )
        self.delete = to_raw_response_wrapper(
            contracts.delete,
        )
        self.end_date_billing_entities = to_raw_response_wrapper(
            contracts.end_date_billing_entities,
        )


class AsyncContractsResourceWithRawResponse:
    def __init__(self, contracts: AsyncContractsResource) -> None:
        self._contracts = contracts

        self.create = async_to_raw_response_wrapper(
            contracts.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            contracts.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            contracts.update,
        )
        self.list = async_to_raw_response_wrapper(
            contracts.list,
        )
        self.delete = async_to_raw_response_wrapper(
            contracts.delete,
        )
        self.end_date_billing_entities = async_to_raw_response_wrapper(
            contracts.end_date_billing_entities,
        )


class ContractsResourceWithStreamingResponse:
    def __init__(self, contracts: ContractsResource) -> None:
        self._contracts = contracts

        self.create = to_streamed_response_wrapper(
            contracts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            contracts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            contracts.update,
        )
        self.list = to_streamed_response_wrapper(
            contracts.list,
        )
        self.delete = to_streamed_response_wrapper(
            contracts.delete,
        )
        self.end_date_billing_entities = to_streamed_response_wrapper(
            contracts.end_date_billing_entities,
        )


class AsyncContractsResourceWithStreamingResponse:
    def __init__(self, contracts: AsyncContractsResource) -> None:
        self._contracts = contracts

        self.create = async_to_streamed_response_wrapper(
            contracts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            contracts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            contracts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            contracts.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            contracts.delete,
        )
        self.end_date_billing_entities = async_to_streamed_response_wrapper(
            contracts.end_date_billing_entities,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal

import httpx

from ..types import bill_job_list_params, bill_job_create_params, bill_job_recalculate_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.bill_job_response import BillJobResponse
from ..types.shared_params.currency_conversion import CurrencyConversion

__all__ = ["BillJobsResource", "AsyncBillJobsResource"]


class BillJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BillJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BillJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return BillJobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        bill_date: Union[str, date] | NotGiven = NOT_GIVEN,
        bill_frequency_interval: int | NotGiven = NOT_GIVEN,
        billing_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY", "AD_HOC"] | NotGiven = NOT_GIVEN,
        currency_conversions: Iterable[CurrencyConversion] | NotGiven = NOT_GIVEN,
        day_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_invoice_date: Union[str, date] | NotGiven = NOT_GIVEN,
        last_date_in_billing_period: Union[str, date] | NotGiven = NOT_GIVEN,
        month_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        target_currency: str | NotGiven = NOT_GIVEN,
        timezone: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        week_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        year_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Create a new BillJob to handle asynchronous bill calculations for a specific
        Organization.

        This operation allows you to initiate the processing of bills according to
        specified parameters. For example, create a BillJob to run only those bills
        where `billingFrequency` is `MONTHLY`. Note that if you want to run a BillJob
        for all billing frequencies, simply omit the `billingFrequency` request
        parameter.

        Once created, the BillJob's progress can be monitored:

        - In the Running Tasks panel in the m3ter Console - for more details, see
          [Running Bills Manually](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/running-bills-and-viewing-bill-details#running-bills-manually)
        - Queried using the
          [List BillJobs](https://www.m3ter.com/docs/api#tag/BillJob/operation/ListBillJobs)
          operation.

        **NOTES:**

        - **Consolidated bills**. If you've already run billing with the Consolidate
          bills option disabled for your Organization but you then enable it, subsequent
          Bills for specific bill dates will now start afresh and not update earlier
          non-consolidated Bills for the same bill date. To avoid any billing conflicts,
          you might want to archive these earlier versions or delete them entirely.
        - **Maximum concurrent BillJobs**. If you already have 10 BillJobs currently
          running, and try to create another one, you'll get an HTTP 429 response (Too
          many requests). When one of the existing BillJobs has completed, you'll be
          able to submit another job

        Args:
          account_ids: An array of UUIDs representing the end customer Accounts associated with the
              BillJob.

          bill_date: The specific billing date _(in ISO 8601 format)_, determining when the Bill was
              generated.

              For example: `"2023-01-24"`.

          bill_frequency_interval: How often Bills are issued - used in conjunction with `billingFrequency`.

              For example, if `billingFrequency` is set to Monthly and `billFrequencyInterval`
              is set to 3, Bills are issued every three months.

          billing_frequency: How often Bills are generated.

              - **Daily**. Starting at midnight each day, covering a twenty-four hour period
                following.

              - **Weekly**. Starting at midnight on a Monday morning covering the seven-day
                period following.

              - **Monthly**. Starting at midnight on the morning of the first day of each
                month covering the entire calendar month following.

              - **Annually**. Starting at midnight on the morning of the first day of each
                year covering the entire calendar year following.

              - **Ad_Hoc**. Use this setting when a custom billing schedule is used for
                billing an Account, such as for billing of Prepayment/Commitment fees using a
                custom billing schedule.

          currency_conversions: An array of currency conversion rates from Bill currency to Organization
              currency. For example, if Account is billed in GBP and Organization is set to
              USD, Bill line items are calculated in GBP and then converted to USD using the
              defined rate.

          day_epoch: The starting date _(epoch)_ for Daily billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for daily Bills.

          due_date: The due date _(in ISO 8601 format)_ for payment of the Bill.

              For example: `"2023-02-24"`.

          external_invoice_date: For accounting purposes, the date set at Organization level to use for external
              invoicing with respect to billing periods - two options:

              - `FIRST_DAY_OF_NEXT_PERIOD` _(Default)_. Used when you want to recognize usage
                revenue in the following period.
              - `LAST_DAY_OF_ARREARS`. Used when you want to recognize usage revenue in the
                same period that it's consumed, instead of in the following period.

              For example, if the retrieved Bill was on a monthly billing frequency and the
              billing period for the Bill is September 2023 and the _External invoice date_ is
              set at `FIRST_DAY_OF_NEXT_PERIOD`, then the `externalInvoiceDate` will be
              `"2023-10-01"`.

              **NOTE:** To change the `externalInvoiceDate` setting for your Organization, you
              can use the
              [Update OrganizationConfig](https://www.m3ter.com/docs/api#tag/OrganizationConfig/operation/GetOrganizationConfig)
              call.

          last_date_in_billing_period: Specifies the date _(in ISO 8601 format)_ of the last day in the billing period,
              defining the time range for the associated Bills.

              For example: `"2023-03-24"`.

          month_epoch: The starting date _(epoch)_ for Monthly billing frequency _(in ISO 8601
              format)_, determining the first Bill date for monthly Bills.

          target_currency: The currency code used for the Bill, such as USD, GBP, or EUR.

          timezone: Specifies the time zone used for the generated Bills, ensuring alignment with
              the local time zone.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          week_epoch: The starting date _(epoch)_ for Weekly billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for weekly Bills.

          year_epoch: The starting date _(epoch)_ for Yearly billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for yearly Bills.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/billjobs",
            body=maybe_transform(
                {
                    "account_ids": account_ids,
                    "bill_date": bill_date,
                    "bill_frequency_interval": bill_frequency_interval,
                    "billing_frequency": billing_frequency,
                    "currency_conversions": currency_conversions,
                    "day_epoch": day_epoch,
                    "due_date": due_date,
                    "external_invoice_date": external_invoice_date,
                    "last_date_in_billing_period": last_date_in_billing_period,
                    "month_epoch": month_epoch,
                    "target_currency": target_currency,
                    "timezone": timezone,
                    "version": version,
                    "week_epoch": week_epoch,
                    "year_epoch": year_epoch,
                },
                bill_job_create_params.BillJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Retrieve a Bill Job for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/billjobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: str | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[BillJobResponse]:
        """
        Retrieve a list of BillJobs.

        This endpoint retrieves a list of BillJobs for a specified organization. The
        list can be paginated for easier management, and allows you to query and filter
        based on various parameters, such as BillJob `status` and whether or not BillJob
        remains `active`.

        Args:
          active: Boolean filter to retrieve only active BillJobs and exclude completed or
              cancelled BillJobs from the results.

              - TRUE - only active BillJobs.
              - FALSE - all BillJobs including completed and cancelled BillJobs.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              BillJobs in a paginated list.

          page_size: Specifies the maximum number of BillJobs to retrieve per page.

          status: Filter BillJobs by specific status. Allows for targeted retrieval of BillJobs
              based on their current processing status.

              Possible states are:

              - PENDING
              - INITIALIZING
              - RUNNING
              - COMPLETE
              - CANCELLED

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/billjobs",
            page=SyncCursor[BillJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    bill_job_list_params.BillJobListParams,
                ),
            ),
            model=BillJobResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Cancel an ongoing BillJob for the given Organization and BillJob UUID.

        This endpoint allows you to halt the processing of a specific BillJob, which
        might be necessary if there are changes in billing requirements or other
        operational considerations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/organizations/{org_id}/billjobs/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    def recalculate(
        self,
        *,
        org_id: str | None = None,
        bill_ids: List[str],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Create a new BillJob specifically to recalculate existing bills for a given
        Organization.

        This operation is essential when adjustments or corrections are required in
        previously calculated bills. The recalculated bills when the BillJob is complete
        can be checked in the m3ter Console Bill Management page or queried by using the
        [List Bills](https://www.m3ter.com/docs/api#tag/Bill/operation/ListBills)
        operation.

        **NOTE:**

        - **Response Schema**. The response schema for this call is dynamic. This means
          that the response might not contain all of the parameters listed. If set to
          null,the parameter is hidden to help simplify the output as well as to reduce
          its size and improve performance.

        Args:
          bill_ids: The array of unique identifiers (UUIDs) for the Bills which are to be
              recalculated.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/billjobs/recalculate",
            body=maybe_transform(
                {
                    "bill_ids": bill_ids,
                    "version": version,
                },
                bill_job_recalculate_params.BillJobRecalculateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )


class AsyncBillJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBillJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncBillJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        bill_date: Union[str, date] | NotGiven = NOT_GIVEN,
        bill_frequency_interval: int | NotGiven = NOT_GIVEN,
        billing_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY", "AD_HOC"] | NotGiven = NOT_GIVEN,
        currency_conversions: Iterable[CurrencyConversion] | NotGiven = NOT_GIVEN,
        day_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_invoice_date: Union[str, date] | NotGiven = NOT_GIVEN,
        last_date_in_billing_period: Union[str, date] | NotGiven = NOT_GIVEN,
        month_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        target_currency: str | NotGiven = NOT_GIVEN,
        timezone: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        week_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        year_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Create a new BillJob to handle asynchronous bill calculations for a specific
        Organization.

        This operation allows you to initiate the processing of bills according to
        specified parameters. For example, create a BillJob to run only those bills
        where `billingFrequency` is `MONTHLY`. Note that if you want to run a BillJob
        for all billing frequencies, simply omit the `billingFrequency` request
        parameter.

        Once created, the BillJob's progress can be monitored:

        - In the Running Tasks panel in the m3ter Console - for more details, see
          [Running Bills Manually](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/running-bills-and-viewing-bill-details#running-bills-manually)
        - Queried using the
          [List BillJobs](https://www.m3ter.com/docs/api#tag/BillJob/operation/ListBillJobs)
          operation.

        **NOTES:**

        - **Consolidated bills**. If you've already run billing with the Consolidate
          bills option disabled for your Organization but you then enable it, subsequent
          Bills for specific bill dates will now start afresh and not update earlier
          non-consolidated Bills for the same bill date. To avoid any billing conflicts,
          you might want to archive these earlier versions or delete them entirely.
        - **Maximum concurrent BillJobs**. If you already have 10 BillJobs currently
          running, and try to create another one, you'll get an HTTP 429 response (Too
          many requests). When one of the existing BillJobs has completed, you'll be
          able to submit another job

        Args:
          account_ids: An array of UUIDs representing the end customer Accounts associated with the
              BillJob.

          bill_date: The specific billing date _(in ISO 8601 format)_, determining when the Bill was
              generated.

              For example: `"2023-01-24"`.

          bill_frequency_interval: How often Bills are issued - used in conjunction with `billingFrequency`.

              For example, if `billingFrequency` is set to Monthly and `billFrequencyInterval`
              is set to 3, Bills are issued every three months.

          billing_frequency: How often Bills are generated.

              - **Daily**. Starting at midnight each day, covering a twenty-four hour period
                following.

              - **Weekly**. Starting at midnight on a Monday morning covering the seven-day
                period following.

              - **Monthly**. Starting at midnight on the morning of the first day of each
                month covering the entire calendar month following.

              - **Annually**. Starting at midnight on the morning of the first day of each
                year covering the entire calendar year following.

              - **Ad_Hoc**. Use this setting when a custom billing schedule is used for
                billing an Account, such as for billing of Prepayment/Commitment fees using a
                custom billing schedule.

          currency_conversions: An array of currency conversion rates from Bill currency to Organization
              currency. For example, if Account is billed in GBP and Organization is set to
              USD, Bill line items are calculated in GBP and then converted to USD using the
              defined rate.

          day_epoch: The starting date _(epoch)_ for Daily billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for daily Bills.

          due_date: The due date _(in ISO 8601 format)_ for payment of the Bill.

              For example: `"2023-02-24"`.

          external_invoice_date: For accounting purposes, the date set at Organization level to use for external
              invoicing with respect to billing periods - two options:

              - `FIRST_DAY_OF_NEXT_PERIOD` _(Default)_. Used when you want to recognize usage
                revenue in the following period.
              - `LAST_DAY_OF_ARREARS`. Used when you want to recognize usage revenue in the
                same period that it's consumed, instead of in the following period.

              For example, if the retrieved Bill was on a monthly billing frequency and the
              billing period for the Bill is September 2023 and the _External invoice date_ is
              set at `FIRST_DAY_OF_NEXT_PERIOD`, then the `externalInvoiceDate` will be
              `"2023-10-01"`.

              **NOTE:** To change the `externalInvoiceDate` setting for your Organization, you
              can use the
              [Update OrganizationConfig](https://www.m3ter.com/docs/api#tag/OrganizationConfig/operation/GetOrganizationConfig)
              call.

          last_date_in_billing_period: Specifies the date _(in ISO 8601 format)_ of the last day in the billing period,
              defining the time range for the associated Bills.

              For example: `"2023-03-24"`.

          month_epoch: The starting date _(epoch)_ for Monthly billing frequency _(in ISO 8601
              format)_, determining the first Bill date for monthly Bills.

          target_currency: The currency code used for the Bill, such as USD, GBP, or EUR.

          timezone: Specifies the time zone used for the generated Bills, ensuring alignment with
              the local time zone.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          week_epoch: The starting date _(epoch)_ for Weekly billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for weekly Bills.

          year_epoch: The starting date _(epoch)_ for Yearly billing frequency _(in ISO 8601 format)_,
              determining the first Bill date for yearly Bills.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/billjobs",
            body=await async_maybe_transform(
                {
                    "account_ids": account_ids,
                    "bill_date": bill_date,
                    "bill_frequency_interval": bill_frequency_interval,
                    "billing_frequency": billing_frequency,
                    "currency_conversions": currency_conversions,
                    "day_epoch": day_epoch,
                    "due_date": due_date,
                    "external_invoice_date": external_invoice_date,
                    "last_date_in_billing_period": last_date_in_billing_period,
                    "month_epoch": month_epoch,
                    "target_currency": target_currency,
                    "timezone": timezone,
                    "version": version,
                    "week_epoch": week_epoch,
                    "year_epoch": year_epoch,
                },
                bill_job_create_params.BillJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Retrieve a Bill Job for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/billjobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: str | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[BillJobResponse, AsyncCursor[BillJobResponse]]:
        """
        Retrieve a list of BillJobs.

        This endpoint retrieves a list of BillJobs for a specified organization. The
        list can be paginated for easier management, and allows you to query and filter
        based on various parameters, such as BillJob `status` and whether or not BillJob
        remains `active`.

        Args:
          active: Boolean filter to retrieve only active BillJobs and exclude completed or
              cancelled BillJobs from the results.

              - TRUE - only active BillJobs.
              - FALSE - all BillJobs including completed and cancelled BillJobs.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              BillJobs in a paginated list.

          page_size: Specifies the maximum number of BillJobs to retrieve per page.

          status: Filter BillJobs by specific status. Allows for targeted retrieval of BillJobs
              based on their current processing status.

              Possible states are:

              - PENDING
              - INITIALIZING
              - RUNNING
              - COMPLETE
              - CANCELLED

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/billjobs",
            page=AsyncCursor[BillJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    bill_job_list_params.BillJobListParams,
                ),
            ),
            model=BillJobResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Cancel an ongoing BillJob for the given Organization and BillJob UUID.

        This endpoint allows you to halt the processing of a specific BillJob, which
        might be necessary if there are changes in billing requirements or other
        operational considerations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/organizations/{org_id}/billjobs/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )

    async def recalculate(
        self,
        *,
        org_id: str | None = None,
        bill_ids: List[str],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillJobResponse:
        """
        Create a new BillJob specifically to recalculate existing bills for a given
        Organization.

        This operation is essential when adjustments or corrections are required in
        previously calculated bills. The recalculated bills when the BillJob is complete
        can be checked in the m3ter Console Bill Management page or queried by using the
        [List Bills](https://www.m3ter.com/docs/api#tag/Bill/operation/ListBills)
        operation.

        **NOTE:**

        - **Response Schema**. The response schema for this call is dynamic. This means
          that the response might not contain all of the parameters listed. If set to
          null,the parameter is hidden to help simplify the output as well as to reduce
          its size and improve performance.

        Args:
          bill_ids: The array of unique identifiers (UUIDs) for the Bills which are to be
              recalculated.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/billjobs/recalculate",
            body=await async_maybe_transform(
                {
                    "bill_ids": bill_ids,
                    "version": version,
                },
                bill_job_recalculate_params.BillJobRecalculateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillJobResponse,
        )


class BillJobsResourceWithRawResponse:
    def __init__(self, bill_jobs: BillJobsResource) -> None:
        self._bill_jobs = bill_jobs

        self.create = to_raw_response_wrapper(
            bill_jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            bill_jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            bill_jobs.list,
        )
        self.cancel = to_raw_response_wrapper(
            bill_jobs.cancel,
        )
        self.recalculate = to_raw_response_wrapper(
            bill_jobs.recalculate,
        )


class AsyncBillJobsResourceWithRawResponse:
    def __init__(self, bill_jobs: AsyncBillJobsResource) -> None:
        self._bill_jobs = bill_jobs

        self.create = async_to_raw_response_wrapper(
            bill_jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            bill_jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            bill_jobs.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            bill_jobs.cancel,
        )
        self.recalculate = async_to_raw_response_wrapper(
            bill_jobs.recalculate,
        )


class BillJobsResourceWithStreamingResponse:
    def __init__(self, bill_jobs: BillJobsResource) -> None:
        self._bill_jobs = bill_jobs

        self.create = to_streamed_response_wrapper(
            bill_jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            bill_jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            bill_jobs.list,
        )
        self.cancel = to_streamed_response_wrapper(
            bill_jobs.cancel,
        )
        self.recalculate = to_streamed_response_wrapper(
            bill_jobs.recalculate,
        )


class AsyncBillJobsResourceWithStreamingResponse:
    def __init__(self, bill_jobs: AsyncBillJobsResource) -> None:
        self._bill_jobs = bill_jobs

        self.create = async_to_streamed_response_wrapper(
            bill_jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            bill_jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            bill_jobs.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            bill_jobs.cancel,
        )
        self.recalculate = async_to_streamed_response_wrapper(
            bill_jobs.recalculate,
        )
